/*******************************************************************************
**
**  PROJECT NAME : OpenGL ES (GLUT32) compatibled code main
**  
**  (C)Copyright2005 , Rage freeware
**  (C)Copyright2005 , Min Chul , Kim ( rageworx@gmail.com )
**
**  All Program codes protects by copyright-law of international. 
**
**  Update [2005-11-11] :: mckim
**  Update [2005-12-27] :: mckim
**
********************************************************************************/

#ifdef _WIN32

#include <windows.h>
#include <stdio.h>
#include <stdlib.h>
#include <math.h>
#include <gl/glut.h>
#include "main.h"
#include "app.h"

// checking Frames per second.
int  checkFPS = 1;

// global function for window surface info
RECT surface_rect;

// global window handle.
HWND ghWnd;

/**************************
 * Function Declarations
 **************************/
LRESULT CALLBACK WndProc (HWND hWnd, UINT message,WPARAM wParam, LPARAM lParam);
void EnableOpenGL (HWND hWnd, HDC *hDC, HGLRC *hRC);
void DisableOpenGL (HWND hWnd, HDC hDC, HGLRC hRC);

/**************************
 * Extern from app.c
 **************************/
extern void InitGL(void);
extern void UpdateGL(void);
extern void DrawGL(void);
extern void KeyControl(int key);
extern void CloseGL(void);

/**************************
 * WinMain
 **************************/
int WINAPI WinMain (HINSTANCE hInstance,HINSTANCE hPrevInstance,
                    LPSTR lpCmdLine,int iCmdShow)
{
    WNDCLASS wc;
    HWND hWnd;
    HDC hDC;
    HGLRC hRC;        
    MSG msg;
    BOOL bQuit = FALSE;

    unsigned int    preTick;
    unsigned int    nexTick;
    int             TickCount;
    unsigned int    framecount = 0;
    float           pre_fps;
    float           fps;
    int             intfps;
    char            fps_str[40] = {0};    


    /* register window class */
    wc.style = CS_OWNDC;
    wc.lpfnWndProc = WndProc;
    wc.cbClsExtra = 0;
    wc.cbWndExtra = 0;
    wc.hInstance = hInstance;
    wc.hIcon = LoadIcon (NULL, IDI_APPLICATION);
    wc.hCursor = LoadCursor (NULL, IDC_ARROW);
    wc.hbrBackground = (HBRUSH) GetStockObject (BLACK_BRUSH);
    wc.lpszMenuName = NULL;
    wc.lpszClassName = DEF_APP_CLASS_NAME;
    RegisterClass (&wc);

    /* create main window */
    hWnd = CreateWindow (
      DEF_APP_CLASS_NAME, DEF_APP_TITLE_NAME, 
      WS_CAPTION | WS_POPUPWINDOW | WS_VISIBLE,
      0, 0, DEF_APP_FORM_WIDTH , DEF_APP_FORM_HEIGHT ,
      NULL, NULL, hInstance, NULL);

    /* adjust client rectangle size */
    {
        DWORD   dwStyle;
        int     xp;
        int     yp;
        int     menuFlag;

        /* change window style to popup */
        dwStyle = GetWindowLong(ghWnd, GWL_STYLE);

        GetWindowRect(ghWnd, &surface_rect);
        //xp = surface_rect.left;
        //yp = surface_rect.top;
		xp = 0;
		yp = 0;

        menuFlag    = GetMenu(ghWnd) != NULL;
        surface_rect.top      = surface_rect.left = 0;
        surface_rect.right    = DEF_APP_FORM_WIDTH;
        surface_rect.bottom   = DEF_APP_FORM_HEIGHT;
        AdjustWindowRect(&surface_rect, dwStyle, menuFlag);

        SetWindowPos(ghWnd, 
					 HWND_NOTOPMOST, 
					 xp, 
					 yp, 
					 surface_rect.right - surface_rect.left, 
					 surface_rect.bottom - surface_rect.top,
                     SWP_NOZORDER | SWP_NOACTIVATE);
    }
    
    /* enable OpenGL for the window */
    EnableOpenGL (hWnd, &hDC, &hRC);

    InitGL();
    //MessageBox(0,"Init_Complete","info",0);
    
    /* program main loop */
    while (!bQuit)
    {
        /* check for messages */
        if (PeekMessage (&msg, NULL, 0, 0, PM_REMOVE))
        {
            /* handle or dispatch messages */
            if (msg.message == WM_QUIT)
            {
                bQuit = TRUE;
            }
            else
            {
                TranslateMessage (&msg);
                DispatchMessage (&msg);
            }
        }
        else
        {

        	preTick = GetTickCount();
            
			/////////////////////////////////////////////////////////////////
			UpdateGL();
            DrawGL();
			/////////////////////////////////////////////////////////////////

            SwapBuffers (hDC);
            Sleep (1);
            nexTick = GetTickCount();
            if ((nexTick - preTick) > 0)  TickCount = nexTick - preTick;

            fps = 1000 / TickCount;
            if (fps > DEF_APP_MAX_FPS) fps = DEF_APP_MAX_FPS;
            
            intfps = (int)fps;
            framecount++;
            
            if (checkFPS)
            {
                sprintf(fps_str,"Flip = 0x%08x | FPS = %03d",framecount,intfps);
                SetWindowText(hWnd,fps_str);
            }
            else
            {
                SetWindowText(hWnd,DEF_APP_TITLE_NAME);
            }
            
            pre_fps = fps;        
		}
    }
    
    CloseGL();

    /* shutdown OpenGL */
    DisableOpenGL (hWnd, hDC, hRC);

    /* destroy the window explicitly */
    DestroyWindow (hWnd);

    return msg.wParam;
}


/********************
 * Window Procedure
 *
 ********************/

LRESULT CALLBACK WndProc (HWND hWnd, UINT message,
                          WPARAM wParam, LPARAM lParam)
{

    switch (message)
    {
		case WM_CREATE:
			return 0;
		case WM_CLOSE:
			PostQuitMessage (0);
			return 0;

		case WM_DESTROY:
			return 0;

		case WM_KEYDOWN:
			if (wParam == VK_ESCAPE)
			{
				PostQuitMessage(0);
				return 0;
			}
			else
			if (wParam == VK_F12)
			{
                checkFPS = !checkFPS;
            }
			else KeyControl(wParam);
			return 0;

		default:
			return DefWindowProc (hWnd, message, wParam, lParam);
    }
}


/**************************************************************
 * Enable OpenGL to Window surface
 *************************************************************/
void EnableOpenGL (HWND hWnd, HDC *hDC, HGLRC *hRC)
{
    PIXELFORMATDESCRIPTOR pfd;
    int iFormat;

    /* get the device context (DC) */
    *hDC = GetDC (hWnd);

    /* set the pixel format for the DC */
    ZeroMemory (&pfd, sizeof (pfd));
    pfd.nSize = sizeof (pfd);
    pfd.nVersion = 1;
    pfd.dwFlags = PFD_DRAW_TO_WINDOW | PFD_SUPPORT_OPENGL | PFD_DOUBLEBUFFER;
    pfd.iPixelType = PFD_TYPE_RGBA;
    pfd.cColorBits = DEF_APP_OPENGL_BPP;
    pfd.cDepthBits = DEF_APP_OPENGL_ZBUFFER;
    pfd.iLayerType = PFD_MAIN_PLANE;
    iFormat = ChoosePixelFormat (*hDC, &pfd);
    SetPixelFormat (*hDC, iFormat, &pfd);

    /* create and enable the render context (RC) */
    *hRC = wglCreateContext( *hDC );
    wglMakeCurrent( *hDC, *hRC );

}


/******************
 * Disable OpenGL
 ******************/

void DisableOpenGL (HWND hWnd, HDC hDC, HGLRC hRC)
{
    wglMakeCurrent (NULL, NULL);
    wglDeleteContext (hRC);
    ReleaseDC (hWnd, hDC);
}
#endif // of _WIN32
